package logger

import (
	"io"
	"os"
	"path"

	"github.com/sirupsen/logrus"
	"github.com/x-cray/logrus-prefixed-formatter"
	"gopkg.in/natefinch/lumberjack.v2"
)

var (
	logger    = logrus.StandardLogger()
	rotaters  []*lumberjack.Logger
	logDir    string
	formatter = prefixed.TextFormatter{}
)

type Config struct {
	FileName  string
	MaxSizeMb int
	MaxBackup int
	MaxDay    int
}

type Logger interface {
	Debugf(format string, args ...interface{})
	Infof(format string, args ...interface{})
	Printf(format string, args ...interface{})
	Warnf(format string, args ...interface{})
	Warningf(format string, args ...interface{})
	Errorf(format string, args ...interface{})
	Fatalf(format string, args ...interface{})
	Panicf(format string, args ...interface{})
	Debug(args ...interface{})
	Info(args ...interface{})
	Print(args ...interface{})
	Warn(args ...interface{})
	Warning(args ...interface{})
	Error(args ...interface{})
	Fatal(args ...interface{})
	Panic(args ...interface{})
	Debugln(args ...interface{})
	Infoln(args ...interface{})
	Println(args ...interface{})
	Warnln(args ...interface{})
	Warningln(args ...interface{})
	Errorln(args ...interface{})
	Fatalln(args ...interface{})
	Panicln(args ...interface{})
}

func init() {
	logger.Formatter = &formatter
}
func LoggerIsStd() bool {
	return !formatter.DisableColors
}
func InitLogger(verbose bool, logDirArg string, config *Config) {
	logDir = logDirArg
	colorSupport, writer := NewLogWriter(config)
	formatter.DisableColors = !colorSupport
	logger.Out = writer
	if verbose {
		logger.Level = logrus.DebugLevel
	} else {
		logger.Level = logrus.InfoLevel
	}
}

func NewLogger(prefix string) Logger {
	return logrus.NewEntry(logger).WithField("prefix", prefix)
}
func NewLogWriter(config *Config) (bool, io.Writer) {
	switch config.FileName {
	case "Stdout":
		return true, os.Stdout
	case "Stderr":
		return true, os.Stderr
	default:
		logpath := config.FileName
		if logDir != "" {
			logpath = path.Join(logDir, config.FileName)
		}
		logger.Info(" Attention!! log writes to ", config.FileName)
		rotater := &lumberjack.Logger{
			Filename:   logpath,
			MaxSize:    config.MaxSizeMb, // megabytes
			MaxBackups: config.MaxBackup,
			MaxAge:     config.MaxDay, //days
			LocalTime:  true,
			Compress:   true,
		}
		rotaters = append(rotaters, rotater)
		return false, rotater
	}
}

func RotateLogger() {
	if len(rotaters) == 0 {
		return
	}
	logger.Info("rotating logger")
	for _, rotater := range rotaters {
		rotater.Rotate()
	}
	logger.Info("rotated")
}
